@@ -5,11 +5,11 @@ def dense_sparse_matmul(denV: tf.Tensor, spW: tf.SparseTensor) -> tf.Tensor:
     """Multiply row vector with sparse matrix
 
     Parameters:
-        denV (tf.Tensor): Dense 1xN row vector.
+        denV (tf.Tensor): Dense BxN or BxTxN dense tensor.
         spW (tf.SparseTensor): Sparse NxM matrix.
 
     Returns:
-        tf.Tensor: Dense 1xM row vector
+        tf.Tensor: Dense BxM or BxTxM dense tensor
 
     Motivation:
         TF only supports multiplying a sparse matrix with a column vector.
@@ -20,6 +20,21 @@ def dense_sparse_matmul(denV: tf.Tensor, spW: tf.SparseTensor) -> tf.Tensor:
     # reshape to list of row vectors if neccessary
     if denV.shape.ndims == 1:
         denV = tf.reshape(denV, (1, -1))
+
     # transpose -> multiply -> transpose back
-    return tf.transpose(tf.sparse.sparse_dense_matmul(
-        tf.sparse.transpose(spW), tf.transpose(denV)))
+    if denV.shape.ndims == 2:
+        # W * V_(batch, dim)
+        return tf.transpose(tf.sparse.sparse_dense_matmul(
+            tf.sparse.transpose(spW, perm=[1, 0]),
+            tf.transpose(denV, perm=[1, 0])))
+
+    if denV.shape.ndims == 3:
+        # W * V_(batch, seqlen, dim)
+        Wt = tf.sparse.transpose(spW, perm=[1, 0])
+        ht = tf.transpose(denV, perm=[0, 2, 1])
+        out = []
+        for ex in ht:
+            out.append(tf.sparse.sparse_dense_matmul(Wt, ex))
+        return tf.transpose(tf.stack(out), perm=[0, 2, 1])
+
+    raise ValueError("Invalid shape: {}".format(denV.shape))
